/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/

//******************************************************************************
//   MSP430FR413x Demo - LPM3.5, Display RTC on LCD in LPM3.5 mode.
//
//   Description: Displays a clock (hour, minute, second) on the LCD display.
//                f(LCD) = 32768Hz/((7+1)*64) = 256Hz.
//                The device enters LPM3.5 mode for ultra low power.
//
//          NOTE: This demo uses the display on the MSP-EXP430FR4133 LaunchPad
//
//                ACLK = XT1 = 32768Hz,
//                MCLK = SMCLK = default DCODIV ~1MHz.
//
//                MSP430FR4133
//             -----------------
//         /|\|                 |
//          | |              XIN|--
// GND      --|RST              |  ~32768Hz
//  |         |             XOUT|--
//  |--0.1uF--|R13              |
//  |--0.1uF--|R23      (L3)COM3|----------------|
//  |--0.1uF--|R33      (L2)COM2|---------------||
//          --|LCDC2    (L1)COM1|--------------|||
//       1uF  |         (L0)COM0|-------------||||
//          --|LCDC1            |    -------------
//            |           L4~L39|---| 1 2 3 4 5 6 |
//            |   except L27~L35|    -------------
//            |                 |       TI LCD
//            |                 |
// Toggle P1.0 using software
//  
//   modified by: Peter Spevak
//                Texas Instruments Deutschland GmbH
//   last change: November 24th 2015
//
//   11/24/2015 adding capacitive touch functionality
//   09/01/2015 adaptation to FR4133 HCA design
//
//   modified by: Milen Stefanov
//                Texas Instruments Deutschland GmbH
//     added: LPM3, FR4133 UART, LCD display of RTC, RTC using XT1
//   last change: October 18th 2015
//
//   Built with IAR Embedded Workbench v6.40.1
//******************************************************************************
#include <msp430.h>
#include <stdint.h>      
#include <stdbool.h> //18.9.2015
//#include <uart.h>


#define a       0x01                            //  AAAA
#define b       0x02                            // F    B
#define c       0x04                            // F    B
#define DOTSEG  0x08                            //  GGGG
#define f  		0x10                            // E    C
#define g       0x20                            // E    C
#define e       0x40                            //  DDDD
#define d       0x80
#define NEGSEG  0x20

const uint8_t LCD_Tab[] = {
		a + b + c + d + e + f,			// Displays "0"
		b + c,							// Displays "1"
		a + b + d + e + g,				// Displays "2"
		a + b + c + d + g,				// Displays "3"
		b + c + f + g,					// Displays "4"
		a + c + d + f +g,				// Displays "5"
		a + c + d + e + f + g,			// Displays "6"
		a + b + c,						// Displays "7"
		a + b + c + d + e + f + g,		// Displays "8"
		a + b + c + d + f + g,			// Displays "9"
		a + b + c + e + f + g,			// Displays "A"
		c + d + e + f + g,				// Displays "B"
		a + d + e + f,					// Displays "C"
		b + c + d + e + g,				// Displays "D"
		a + d + e + f + g,				// Displays "E"
		a + e + f + g,					// Displays "F"
		a + c + d + e + f,				// Displays "G"
		b + c + e + f + g,				// Displays "H"
		e + f,							// Displays "I"
		b + c + d + e,					// Displays "J"
		b + d + e + f + g,				// Displays "K"
		d + e + f,						// Displays "L"
		a + c + e + g,					// Displays "M"
		a + b + c + e + f,				// Displays "N"
		c + e + g,						// Displays "n"
		c + d + e + g,					// Displays "o"
		a + b + c + d + e + f,			// Displays "O"
		a + b + e + f + g,				// Displays "P"
		a + b + c + f + g,				// Displays "Q"
		e + g,							// Displays "r"
		a + c + d + f +g,				// Displays "S"
		d + e + f + g,					// Displays "t"
		a + e + f,						// Displays "T"
		b + c + d + e + f,				// Displays "U"
		c + d + e,						// Displays "v"
		b + d + f + g,					// Displays "W"
		b + c + d + f + g,				// Displays "Y"
		a + b + d + e + g,				// Displays "Z"
		g,								// Displays "-"
		b,								// Displays "'"
		0								// Displays " "
};
#undef a
#undef b
#undef c
#undef d
#undef e
#undef f
#undef g

#define AA 10
#define BB AA+1
#define CC BB+1
#define DD CC+1
#define EE DD+1
#define FF EE+1
#define GG FF+1
#define HH GG+1
#define II HH+1
#define JJ II+1
#define KK JJ+1
#define LL KK+1
#define mm LL+1
#define NN mm+1
#define nn NN+1
#define oo nn+1
#define OO oo+1
#define PP OO+1
#define QQ PP+1
#define rr QQ+1
#define SS rr+1
#define tt SS+1
#define TT tt+1
#define UU TT+1
#define VV UU+1
#define WW VV+1
#define YY WW+1
#define ZZ YY+1
#define BR ZZ+1
#define DT BR+1
#define SP DT+1

uint8_t count = 0;                          // definition of counter            // 09/01/2015

#define direct

#define TXD BIT3                            // P1.3  Bit bang this pin to       // 10/15/2014 
                                            // send out the data to the tool
//#define TXDOUT P1OUT                        // UART output SFR                // 10/15/2014  // 08/29/2014
#define TXDOUT P1OUT                        // UART output SFR                  // 10/15/2014
//#define TXDDIR P1DIR                        // UART direction SFR             // 10/15/2014  // 08/29/2014
#define TXDIR P1DIR                         // UART direction SFR            

unsigned char RXData = 0;
unsigned char TXData = 1;

volatile unsigned char * Seconds = &BAKMEM0_L;               // Store seconds in the backup RAM module
volatile unsigned char * Minutes = &BAKMEM0_H;               // Store minutes in the backup RAM module
volatile unsigned char * Hours = &BAKMEM1_L;                 // Store hours in the backup RAM module

void CLOCK_Config();                        // initialize clock system          // 09/01/2015
void LCD_Config();                          // initialize LCD                   // 09/01/2015
void LCD_Disp_Test();                       // LCD test function                // 09/01/2015
void LCD_DisplayLongDecimal_A(uint32_t ui32Number, uint8_t ui8Dot);     //LCD Long 18.9.2015
void LCD_DisplayLongDecimal_B(uint16_t ui16Number, uint8_t ui8Dot);
void LCD_DisplayNum(uint8_t Number, uint8_t Position);

void eUSCIA0_Config();
void UART_send_byte(uint8_t data2send);
void data_frame_send(void);
void Inc_RTC();
void initGpio(void);


// UART Port definition
#define UART_PORT_OUT	P1OUT
#define UART_PORT_DIR	P1DIR
#define	UART_PORT_SEL	P1SEL0
#define	UART_BIT_TXD	BIT1            //MS 13. Oct'15
#define	UART_BIT_RXD	BIT0            //MS 13.10.15

//UART globals and buffers
#define TXBUF_SIZE 30
uint8_t ui8TX_buffer[TXBUF_SIZE];
uint8_t ui8TX_counter;
uint8_t ui8TX_number = TXBUF_SIZE;
#define RXBUF_SIZE 10  //4 Bytes from CC1310 = 2 readings for each LMT70 of 2 Bytes (uint16) 
uint8_t ui8RX_buffer[RXBUF_SIZE+1]; //+ "\n"
uint8_t ui8RX_counter=0;
bool bRX_enable = false;
bool bRX_finished = false;

uint16_t lmt70heat = 0;
uint16_t lmt70room = 0;
//uint8_t SENSOR_NUM = 3; //2 x LMT70 + 1x Cap Touch

#define DataHead 0x55AA                     // Synch Byte of TouchPro GUI       // 08/29/2014
                                            // header
#define SENSOR_NUM 2                        // define the number of values      // 10/15/2014 //was 5
                                            // to be transmitted to the GUI
                                            // 8 CT RAW vallues
                                            // 1 selectable Startup Baseline 
                                            //   value
                                            // 1 selectable Current Baseline
                                            //   value
uint16_t GUIdata[SENSOR_NUM] = {0,0};               // array of Capacitive Sensor       // 10/15/2014
                                            // resutls
uint16_t SENSOR_OFF[SENSOR_NUM];            // Sensor offset, to be applied to  // 10/15/2014
                                            // sensor data, transmitted to the 
                                            // TouchPro GUI. Allows shifting
                                            // the count values of sensors with
                                            // different sensitivity into the
                                            // same range

/******************************************************************************
* Capacitive Touch definitions
*
******************************************************************************/
/* Defines WDT SMCLK interval for sensor measurements*/                         // 11/24/2015
#define WDT_meas_setting (DIV_SMCLK_512)                                        // 11/24/2015
/* Defines WDT ACLK interval for delay between measurement cycles*/             // 11/24/2015
#define WDT_delay_setting (DIV_ACLK_512)                                        // 11/24/2015

/* Sensor settings*/
//#define KEY_LVL     220                     // Defines threshold for a key press // 11/24/2015
#define KEY_LVL      600                    // defines threshold for a key      // 11/24/2015
                                            // press
/*Set to ~ half the max delta expected*/

/* Definitions for use with the WDT settings*/
#define DIV_ACLK_32768  (WDT_ADLY_1000)     // ACLK/32768                       // 11/24/2015
#define DIV_ACLK_8192   (WDT_ADLY_250)      // ACLK/8192                        // 11/24/2015
#define DIV_ACLK_512    (WDT_ADLY_16)       // ACLK/512                         // 11/24/2015
#define DIV_ACLK_64     (WDT_ADLY_1_9)      // ACLK/64                          // 11/24/2015
#define DIV_SMCLK_32768 (WDT_MDLY_32)       // SMCLK/32768                      // 11/24/2015
#define DIV_SMCLK_8192  (WDT_MDLY_8)        // SMCLK/8192                       // 11/24/2015
#define DIV_SMCLK_512   (WDT_MDLY_0_5)      // SMCLK/512                        // 11/24/2015
#define DIV_SMCLK_64    (WDT_MDLY_0_064)    // SMCLK/64                         // 11/24/2015

#define CAPTIOPOSEL_0 0x0000 /* PJ.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_1 0x0010 /* P1.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_2 0x0020 /* P2.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_3 0x0030 /* P3.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_4 0x0040 /* P4.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_5 0x0050 /* P5.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_6 0x0060 /* P6.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_7 0x0070 /* P7.x selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPOSEL_8 0x0080 /* P8.x selected for Capacitive Touch           */ // 11/24/2015

#define CAPTIOPISEL_0 0x0000 /* Px.0 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_1 0x0002 /* Px.1 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_2 0x0004 /* Px.2 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_3 0x0006 /* Px.3 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_4 0x0008 /* Px.4 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_5 0x000A /* Px.5 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_6 0x000C /* Px.6 selected for Capacitive Touch           */ // 11/24/2015
#define CAPTIOPISEL_7 0x000E /* Px.7 selected for Capacitive Touch           */ // 11/24/2015

// Global variables for sensing
unsigned int i,j;                           // loop counter variables for CT    // 11/24/2015
unsigned int base_cnt, meas_cnt;                                                // 11/24/2015
int delta_cnt;                                                                  // 11/24/2015
char key_pressed;                                                               // 11/24/2015
/* System Routines*/
void measure_count(void);                   // Measures each capacitive sensor  // 11/24/2015
char LCD_countdown;                         // LCD down counter, defines active // 11/24/2015
                                            // time of LCD after touch event in 
                                            // seconds

int main(void)
{
  
    WDTCTL = WDTPW | WDTHOLD;               // Stop watchdog timer

     if (SYSRSTIV == SYSRSTIV_LPM5WU)                        // If LPM3.5 wakeup
    {
  //      Inc_RTC();                                          // Real clock
        __enable_interrupt();               // The RTC interrupt should trigger now...
        //__bis_SR_register(LPM3_bits | GIE);                 // Re-enter LPM3.5
    }
    else { 
          
    // Device powered up from a cold start.                                                     
    // It configures the device and puts the device into LPM3.5

        // Configure backup memory
 //   *(unsigned int *)BAK_RAM_BASE = 0;
    
    initGpio();                 //used from CCS code example lpm3_5_02.c 12. Oct'15
    
    P2OUT &= ~BIT4;                         // clear P2.4                       // 09/01/2015
    P2DIR |= BIT4;                          // set P2.4 to output               // 09/01/2015
//    P1DIR |= BIT3;                          // set P1.3 to output for UART BitBanging to PC              // 09/23/2015

    //control of TPS62730 Vout
//    P8DIR |= BIT3;                          // set P8.3 to output               done in in initGpio()
      P8OUT &= ~BIT3;                         // clear P8.3 =  BYPASS mode  for TPS62730  = LOW     // 09/15/2015 MS OK or  P8OUT |= BIT3;
      
      //P1.1 is Input (=0=LOW), the rest is HIGH
     P1DIR &= ~UART_BIT_RXD;   //set as Input
     P1REN &= ~UART_BIT_RXD;   //set as Input
     //9. Dec
     P1REN |= UART_BIT_RXD;   //enable Pull-up/down
     P1OUT |= UART_BIT_RXD;  // Pull-up selected
     P1SEL0 &= ~UART_BIT_RXD; //General purpose I/O select
    
    *Seconds = 0;                                       // Set initial time to 12:00:00
    *Minutes = 0;
    *Hours = 12;
#ifdef TESTMODE    
    eUSCIA0_Config();
#endif
    PM5CTL0 &= ~LOCKLPM5;                   // Disable the GPIO power-on 
                                            // default high-impedance mode
                                            // to activate previously 
                                            // configured port settings

    CLOCK_Config();                                                                                                                                // initialize clock system          // 09/01/2015
    
    // Configure RTC
    RTCCTL = RTCSS__XT1CLK + RTCSR + RTCPS__1 | RTCIE;
    RTCMOD = 32768-1;                   // Set RTC modulo to 32768 to trigger interrupt each second
    
    LCD_Config();                           // initialize LCD module            // 09/01/2015

 // LCD_Disp_Test();                        // test LCD                         // 09/01/2015
    
    LCD_DisplayLongDecimal_A(0000000,4);
    //LCD_DisplayLongDecimal_B(430,2);
    LCD_DisplayNum(HH,11);
    LCD_DisplayNum(CC,10);
    LCD_DisplayNum(AA,9);
    
    measure_count();                        // Establish baseline capacitance   // 11/24/2015
    base_cnt = meas_cnt;
    
    for(i=15; i>0; i--)                     // Repeat and avg base measurement  // 11/24/2015
    { 
        measure_count();                                                        // 11/24/2015
        base_cnt = (meas_cnt+base_cnt)/2;                                       // 11/24/2015
    }
    
    while(1) 
    {
      //P2OUT ^= BIT4;                        // toggle P2.4  = LED                    // 09/01/2015
     __bis_SR_register(LPM3_bits|GIE);      // Enter LPM3, works with UART RX! 20/09/2015
     
     j = KEY_LVL;                                                               // 11/24/2015
       key_pressed = 0;                     // Assume no keys are pressed       // 11/24/2015
        
       measure_count();                     // Measure all sensors              // 11/24/2015
        
       delta_cnt = base_cnt - meas_cnt;     // Calculate delta: c_change        // 11/24/2015
        
       /* Handle baseline measurment for a base C decrease*/
       if (delta_cnt < 0)                   // If negative: result increased    // 11/24/2015
       {                                    // beyond baseline, i.e. cap dec    // 11/24/2015
         base_cnt = (base_cnt+meas_cnt) >> 1; 
                                            // Re-average quickly               // 11/24/2015
         delta_cnt = 0;                     // Zero out for pos determination   // 11/24/2015
       }
       if (delta_cnt > j)                   // Determine if each key is pressed // 11/24/2015
       {                                    // per a preset threshold           // 11/24/2015
         j = delta_cnt;                     //                                  // 11/24/2015
         key_pressed = 1;                   // key pressed                      // 11/24/2015
         LCDCTL0 |= LCDON;                  // turn LCD on                      // 11/24/2015
         LCD_countdown = 5;                // set active time to 5 sec         // 12/09/2015
       }
       else                                                                     // 11/24/2015
       {
         key_pressed = 0;                                                       // 11/24/2015
         LCD_countdown--;                   // decrement countdown              // 11/24/2015
         if(LCD_countdown==0)               // if 0                             // 11/24/2015
         {
           LCDCTL0 &= ~LCDON;               // turn LCD off                     // 11/24/2015
         }
       }
       
       /* Delay to next sample */
       //WDTCTL = WDT_delay_setting;             // WDT, ACLK, interval timer     // 11/24/2015
                                            // removed, wakeup generated by RTC
        
       /* Handle baseline measurment for a base C increase*/
       if (!key_pressed)                       // Only adjust baseline down     // 11/24/2015
       {                                       // if no keys are touched
         base_cnt = base_cnt - 1;        // Adjust baseline down, should be     // 11/24/2015
       }                                   // slow to accomodate for genuine   // 11/24/2015
     
     /*
      if  (bRX_finished == 1)        {        
      
        for (uint8_t i=0; i<10; i++) {        
            if (ui8RX_buffer[i] >= 0x30 && ui8RX_buffer[i] <= 0x39) 
               ui8RX_buffer[i] &= 0x0F;        
            else ui8RX_buffer[i] = 0x00;
        }
       //     else if (ui8RX_buffer[ui8RX_counter] == 0x20 && i == 4)
                    //UART character error
        lmt70heat =  ui8RX_buffer[0]*1000 + ui8RX_buffer[1]*100 + ui8RX_buffer[2]*10 +  ui8RX_buffer[3];
                  
      //      else if (ui8RX_buffer[ui8RX_counter] == 0x10 && ui8RX_counter == 9)
                   //UART character error
        lmt70room =  ui8RX_buffer[5]*1000 + ui8RX_buffer[6]*100 + ui8RX_buffer[7]*10 + ui8RX_buffer[8];                               
        
        GUIdata[0] = lmt70heat;
        GUIdata[1] = lmt70room;
        bRX_finished = false; 
       // P2OUT ^= BIT4; 
      
     }            
//        data_frame_send();
       Inc_RTC();
       LCD_DisplayLongDecimal_A(lmt70room,2);
       LCD_DisplayLongDecimal_B(lmt70heat,2);
       
       
    
        PMMCTL0_H = PMMPW_H;                                // Open PMM Registers for write
        PMMCTL0_L |= PMMREGOFF_L;                           // and set PMMREGOFF
    
        __bis_SR_register(LPM3_bits | GIE);                 // Enter LPM3.5
        __no_operation();                                   // For debugger  
*/                                                                              // 11/24/2015        
   }   
   
  }
}

//M.S. 18.9.2015, UART added
//*****************************************************************************
//
//! Perform eUSCI initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void eUSCIA0_Config()
{
        //UART_PORT_SEL |= UART_BIT_TXD + UART_BIT_RXD; // set pin as UART function: RX OK, 8. Oct 
        UART_PORT_SEL |= UART_BIT_TXD; // set pin as UART function: UART_BIT_TXD + 

	UCA0CTLW0 |= UCSWRST;                         // Put eUSCI in reset
	UCA0CTLW0 |= UCSSEL__SMCLK;   // 8-e-1 LSB SMCLK

	UCA0BR0 = 6;                                  // 9600 baud with 1.04Mhz
        	//UCA0BR0 = 26;                                 // 9600 baud with 4MHz SMCLK
	UCA0BR1 = 0;
	UCA0MCTLW = 0x2200 + UCOS16 + UCBRF_13;       // 1.04MHz/9600

        //UCA0MCTLW = 0xB600 + UCOS16 + UCBRF_0;       // 4MHz/9600 from UG4133, Table 14-5
	UCA0CTLW0 &= ~UCSWRST;                        // Initialize eUSCI
	UCA0IE |= UCRXIE;
}

#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
	{
	case USCI_NONE: break;
	case USCI_UART_UCRXIFG:                   
              UCA0IFG &=~ UCRXIFG;            // Clear interrupt
              ui8RX_buffer[ui8RX_counter] = UCA0RXBUF;             // Clear buffer  
              ui8RX_counter++;
            //  UCA0TXBUF = ui8RX_counter;
      
              if (ui8RX_counter == 11) {
                ui8RX_counter = 0;      // reset counter for RX buffer              
                bRX_finished = true;
                // Enter LPM3, works with UART RX! 20/09/2015
                __bic_SR_register_on_exit(LPM3_bits| GIE);            
              }             
              break;
	case USCI_UART_UCTXIFG:
		break;
	case USCI_UART_UCSTTIFG: break;
	case USCI_UART_UCTXCPTIFG: break;
	}
}

#pragma vector = RTC_VECTOR
__interrupt void RTC_ISR(void) {
    switch(__even_in_range(RTCIV, RTCIV_RTCIF))
    {
        case RTCIV_NONE :
            break;

        case RTCIV_RTCIF:
              Inc_RTC();  
              __bic_SR_register_on_exit(LPM3_bits);   // Exit LPM3 on reti      // 11/24/2015
            break;

        default:
            break;
    }
}


/******************************************************************************
* Function calculates data length and check sum. The function is using the 
* the TouchPro GUI format
*
* last change: July 1st 2013
*
* 07/01/2013 changing variable name for GUI data
* 05/27/2013 debugging UART communication
******************************************************************************/
void data_frame_send(void) 
{
  //unsigned char check_sum = 0;                                                // 05/27/2013
  uint8_t check_sum = 0;                    // checksum of the data packet      // 05/27/2013
  //unsigned char i = 0;                                                        // 05/27/2013
  uint8_t i=0;                              //                                  // 05/27/2013
  //unsigned int temp = 0;                                                      // 05/27/2013
  uint16_t temp = 0;                        // data word to be transmitted      // 05/27/2013
  //unsigned char data_length = 0;                                              // 05/27/2013
  uint8_t data_length = 0;                  // specifies data packet length     // 05/27/2013

  //data_length = SENSOR_NUM * 3  + 1;        // # of channels * 3 + 1          // 07/01/2013
  data_length = SENSOR_NUM * 3  + 1;        // # of channels * 3 + 1            // 07/01/2013
                                            // (checksum).
                                                                               // 1 byte for channel number,  
                                            // 1 byte for high order data byte
                                                                               // and 3rd byte is for low order 
                                            // byte.  The plus 1 is for 
                                            // checksum byte.

  temp = DataHead;
  temp >>= 8;                                                //Send high byte
  //UART_send_byte((unsigned char) temp);                                       // 05/27/2013
  UART_send_byte((uint8_t) temp);                                               // 05/27/2013

  //check_sum = (unsigned char) temp;                                           // 05/27/2013
  check_sum = (uint8_t) temp;                                                   // 05/27/2013 

  //UART_send_byte((unsigned char) DataHead);                                   // 05/27/2013
  UART_send_byte((uint8_t) DataHead);                                           // 05/27/2013

  //check_sum += (unsigned char) DataHead;                                      // 05/27/2013
  check_sum += (uint8_t) DataHead;                                              // 05/27/2013

  UART_send_byte(data_length);
  check_sum += data_length;

  for (i = 0; i < SENSOR_NUM; i++) 
  {
    UART_send_byte(i + 1);
    check_sum += i + 1;
    //temp = count[i];                                                          // 07/01/2013
    temp = GUIdata[i];                                                          // 07/01/2013
    temp >>= 8;                                      //Send high byte

    //UART_send_byte((unsigned char) temp);                                     // 05/27/2013
    UART_send_byte((uint8_t) temp);                                             // 05/27/2013
    //check_sum += (unsigned char) temp;                                        // 05/27/2013
    check_sum += (uint8_t) temp;                                                // 05/27/2013

    //UART_send_byte((unsigned char) count[i]);                                 // 05/27/2013
    //UART_send_byte((uint8_t) count[i]);                                       // 07/01/2013  // 05/27/2013
    UART_send_byte((uint8_t) GUIdata[i]);                                       // 07/01/2013
    //check_sum += (unsigned char) count[i];                                    // 05/27/2013
    //check_sum += (uint8_t) count[i];                                          // 07/01/2013  // 05/27/2013
    check_sum += (uint8_t) GUIdata[i];                                          // 07/01/2013
  }
  //UART_send_byte((unsigned char) check_sum);                                  // 05/27/2013
  UART_send_byte((uint8_t) check_sum);                                          // 05/27/2013
}

/******************************************************************************
* Function sends the data as low byte and high byte using bit banging method  
* on P1.3, P1.3 is either cleared or set to send each value of count array
*
* The timing is generated by a CPU delay loop, based on the MCLK set to factory
* calibration data for 1MHz DCO frequency!
*
* last change: August 9th 2013
*
* 08/09/2013 continued
* 08/08/2013 adding compile option for optocoupler Booster Pack
* 07/01/2013 adaptation to Demo HW
* 05/27/2013 debugging UART communication
******************************************************************************/
//void UART_send_byte(unsigned char data2send)                                  // 05/27/2013
void UART_send_byte(uint8_t data2send)                                          // 05/27/2013
{
  //unsigned int temp = 0;                                                      // 05/27/2013
  uint16_t temp = 0;                        // data word to send                // 05/27/2013
  //unsigned int i;                                                             // 05/27/2013
  uint16_t i;                                                                   // 05/27/2013

  temp = data2send | (0xff00);                         //send low_8_bit
 temp <<= 1;
  for (i = BIT0; i < BITB; i <<= 1) 
  {
    if (temp & i)
    {
#ifdef Opto_Boost                           // in case of use of the            // 08/08/2013
                                            // optocoupler
      TXDOUT &= ~TXD;                       // reset TXD                        // 08/08/2013
#endif                                      //                                  // 08/08/2013
#ifdef direct                               // non-inverted signal in case of   // 08/08/2013
                                            // direct inerconnection
      TXDOUT |= TXD;
#endif                                      //                                  // 08/08/2013
    }
    else
    {
#ifdef Opto_Boost                           // in case of use of the            // 08/08/2013
                                            // optocoupler
      TXDOUT |= TXD;                        // set TXD                          // 08/08/2013
#endif                                      //                                  // 08/08/2013
#ifdef direct                               // non-inverted signal in case of   // 08/08/2013
                                            // direct inerconnection
      TXDOUT &= ~TXD;
#endif                                      //                                  // 08/08/2013
    }
    __delay_cycles(90);                     // Sets data rate to 9600 baud    // 10/22/2014
                                            // based on CALBC1_1MHZ values!
    //__delay_cycles(32);                     //                                  // 10/22/2014
    
  }
#ifdef direct                               // non-inverted signal in case of   // 08/08/2013
                                            // direct inerconnection
  TXDOUT |= TXD;                                               // end of one byte
#endif                                      //                                  // 08/08/2013
#ifdef Opto_Boost                           // in case of use of the            // 08/08/2013
                                            // optocoupler
  TXDOUT &= ~TXD;                           // reset TXD                        // 08/09/2013
#endif                                      //                                  // 08/08/2013
}


//*****************************************************************************
//
//! Perform System clock initialize.
//! ACLK = XT1CLK = 32.738kHz  MCLK = SMCLK = default ~ 1.04MHz
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void CLOCK_Config()
{
	P4SEL0 |= BIT1 + BIT2;                  // Select pin4.1 pin4.2 as crystal function
	do
	{
	  CSCTL7 &= ~(XT1OFFG + DCOFFG);    	// Clear XT1 and DCO fault flag
	  SFRIFG1 &= ~OFIFG;
	}while (SFRIFG1 & OFIFG);             	// Test oscillator fault flag

	CSCTL4 = SELA__XT1CLK;			// IMPORTANT. Select ACLK = XT1 = 32768Hz
        
    //   CSCTL4 = SELA__REFOCLK;
   /*     
        
        //  MSP430FR413x Demo - Configure MCLK for 8MHz and XT1 sourcing ACLK and
//                     FLLREF.
//
//  Description: Configure ACLK = 32768Hz,
//               MCLK = DCO + XT1CLK REF = 8MHz,
//               SMCLK = MCLK/2 = 4MHz.
//               Toggle LED to indicate that the program is running.
        
    __bis_SR_register(SCG0);                // disable FLL
    CSCTL3 |= SELREF__XT1CLK;               // Set XT1CLK as FLL reference source
    CSCTL0 = 0;                             // clear DCO and MOD registers
    CSCTL1 &= ~(DCORSEL_7);                 // Clear DCO frequency select bits first
    CSCTL1 |= DCORSEL_3;                    // Set DCO = 8MHz
    CSCTL2 = FLLD_0 + 243;                  // DCODIV = 8MHz

    __delay_cycles(3);  
    __bic_SR_register(SCG0);                // enable FLL
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1)); // Poll until FLL is locked

    CSCTL4 = SELMS__DCOCLKDIV | SELA__XT1CLK;  // Set ACLK = XT1CLK = 32768Hz
                                            // DCOCLK = MCLK and SMCLK source
    CSCTL5 |= DIVM_0 | DIVS_1;              // MCLK = DCOCLK = 8MHZ,
                                            // SMCLK = MCLK/2 = 4MHz
        
   */     
}

//*****************************************************************************
//
//! Perform LCD initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void LCD_Config()
{
	uint8_t ui8Cnt;
	uint8_t *pLCD = (uint8_t *)&LCDM0;
	for (ui8Cnt = 14; ui8Cnt > 0; ui8Cnt--)			// Clear LCD memory
		*pLCD++ = 0x00;					// CLR LCD

	LCDM0 = BIT7;						// COM0 -- L21
	LCDM1 = BIT6;						// COM1 -- L23
	LCDM2 = BIT5;						// COM2 -- L25
	LCDM3 = BIT4;						// COM3 -- L27

	//SYSCFG2 |= LCDPCTL;      			// R13/R23/R33/LCDC1/LCDC2 pins selected

	LCDPCTL0 = 0xFFFF;
	LCDPCTL1 = 0x0FFF;					// L0~L27 pins selected

	LCDVCTL = LCDSELVDD + LCDCPEN;		// Mode 2
          
	LCDCSSEL0 = 0x00AA;					// Configure COMs and SEGs
	LCDCSSEL1 = 0x0000;

	//LCDCTL0 |= LCD4MUX + LCDLP + LCDON;	// turn on LCD: 4-mux selected  // 11/24/2015
        LCDCTL0 |= LCD4MUX + LCDLP;         // 4-mux selected, LCD off          // 11/24/2015
}

//*****************************************************************************
//
//! Perform Number display.
//!
//
//! \param Number is , Position is
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayNum(uint8_t Number, uint8_t Position)
{
	uint8_t *pLCD = (uint8_t *)&LCDM0;

	if(Position == 0)
	{
		pLCD[13] &= 0x0F;
		pLCD[13] |= (LCD_Tab[Number]&0x0F)<<4;
		pLCD[12] &= 0x0F;
		pLCD[12] |= LCD_Tab[Number]&0xF0;
	}
	else if((Position > 0) && (Position < 8))
	{
		pLCD[15-Position-Position] &= 0xF0;
		pLCD[15-Position-Position] |= (LCD_Tab[Number]&0xF0)>>4;
		pLCD[14-Position-Position] &= 0xF0;
		pLCD[14-Position-Position] |= LCD_Tab[Number]&0x0F;
	}
	else if((Position > 7) && (Position < 12))
	{
		pLCD[Position+Position-11] &= 0x0F;
		pLCD[Position+Position-11] |= (LCD_Tab[Number]&0x0F)<<4;
		pLCD[Position+Position-12] &= 0x0F;
		pLCD[Position+Position-12] |= LCD_Tab[Number]&0xF0;
	}
}

//*****************************************************************************
//
//! Perform Dot display.
//!
//
//! \param Position is
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayDot(uint8_t Position)
{
	uint8_t *pLCD = (uint8_t *)&LCDM0;

	if(Position == 0)
	{
		pLCD[13] |= DOTSEG<<4;
	}
	else if((Position > 0) && (Position < 8))
	{
		pLCD[14-Position-Position] |= DOTSEG;

	}
	else if((Position > 7) && (Position < 12))
	{
		pLCD[Position+Position-11] |= DOTSEG<<4;
	}
}

//*****************************************************************************
//
//! Perform second line display.
//!
//
//! \param
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayLongDecimal_A(uint32_t ui32Number, uint8_t ui8Dot)
{
	uint8_t ui8Cnt;
	uint8_t DispBuff[3];

	for(ui8Cnt=9;ui8Cnt>0;ui8Cnt--)
	{
		DispBuff[9-ui8Cnt]=ui32Number%10;
		ui32Number/=10;
	}
	for(ui8Cnt=7;ui8Cnt>ui8Dot;ui8Cnt--)
	{
		if (DispBuff[ui8Cnt]==0) DispBuff[ui8Cnt]=SP;
		else break;
	}
	for(ui8Cnt=0;ui8Cnt<8;ui8Cnt++)
	{
		LCD_DisplayNum(DispBuff[ui8Cnt],ui8Cnt);
	}

	if(ui8Dot != 0) LCD_DisplayDot(ui8Dot);

#ifdef Chinese_Version
	Display_Chinese_Character();
#endif
}

//*****************************************************************************
//
//! Perform first line display.
//!
//
//! \param Number the number to be displayed; DOT the position of dot
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayLongDecimal_B(uint16_t ui16Number, uint8_t ui8Dot)
{
	uint8_t ui8Cnt;
	uint8_t DispBuff[4];

	for(ui8Cnt=0;ui8Cnt<4;ui8Cnt++)
	{
		DispBuff[ui8Cnt] = ui16Number%10;
		ui16Number /= 10;
	}
	for(ui8Cnt=3;ui8Cnt>ui8Dot;ui8Cnt--)
	{
		if (DispBuff[ui8Cnt]==0) DispBuff[ui8Cnt]=SP;
		else break;
	}
	for(ui8Cnt=8;ui8Cnt<12;ui8Cnt++)
	{
		LCD_DisplayNum(DispBuff[ui8Cnt-8],ui8Cnt);
	}
	LCD_DisplayDot(ui8Dot+8);
}

//*****************************************************************************
//
//! Perform display battery voltage informaiton.
//!
//
//! \param BalVol battery voltage to be displayed
//!
//! \return None.
//
//*****************************************************************************
void LCD_Display_BatteryVoltage(uint16_t ui16BatVol)
{
	LCD_DisplayLongDecimal_A((uint32_t)ui16BatVol*10,3);
	LCD_DisplayNum(VV,7);
	LCD_DisplayNum(oo,6);
	LCD_DisplayNum(LL,5);
	LCD_DisplayNum(SP,4);
	LCD_DisplayNum(VV,0);
	LCD_DisplayNum(SP,8);
	if(ui16BatVol < 280)
	{
		LCD_DisplayNum(LL,11);
		LCD_DisplayNum(oo,10);
		LCD_DisplayNum(WW,9);
	}
	else
	{
		LCD_DisplayNum(nn,11);
		LCD_DisplayNum(oo,10);
		LCD_DisplayNum(rr,9);
	}
}

//*****************************************************************************
//
//! Perform display system error informaiton.
//!
//
//! \param error ID to be displayed
//!
//! \return None.
//
//*****************************************************************************
void LCD_Display_Error(uint8_t ui8Error_ID)
{
	LCD_DisplayNum(BR,8);
	LCD_DisplayNum(BR,9);
	LCD_DisplayNum(BR,10);
	LCD_DisplayNum(BR,11);
	LCD_DisplayLongDecimal_A(ui8Error_ID,0);
	LCD_DisplayNum(EE,7);
	LCD_DisplayNum(rr,6);
	LCD_DisplayNum(rr,5);
	LCD_DisplayNum(oo,4);
	LCD_DisplayNum(rr,3);
}

//*****************************************************************************
//
//! Perform LCD display test.
//! First line "M430"
//! Second line "7654.3210"
//
//! \param None
//!
//! \return None.
//
//*****************************************************************************
void LCD_Disp_Test()
{
	LCD_DisplayNum(5,0);
	LCD_DisplayNum(mm,11);
	LCD_DisplayNum(4,10);
	LCD_DisplayNum(3,9);
	LCD_DisplayNum(0,8);
	LCD_DisplayLongDecimal_A(76543210,1);
	LCD_DisplayNum(0,7);
	LCD_DisplayNum(1,6);
	LCD_DisplayNum(2,5);
	LCD_DisplayNum(3,4);
	LCD_DisplayNum(4,3);
	LCD_DisplayNum(5,2);
	LCD_DisplayNum(6,1);
	LCD_DisplayNum(7,0);
}

// Real clock function
void Inc_RTC()
{
    // Deal with second
    (*Seconds)++;
    *Seconds = (*Seconds) % 60;
    
    //*Seconds =  
    LCD_DisplayNum ((*Seconds) % 10, 0);
    if (((*Seconds) % 10) == 0)
    {
        LCD_DisplayNum ((*Seconds) / 10, 1);
    }
    // Deal with minute
    if ((*Seconds) == 0) 
    {
        (*Minutes)++;  
        *Minutes = (*Minutes) % 60;
        LCD_DisplayNum ((*Minutes) % 10, 2);
        if (((*Minutes) % 10) == 0)
        {
            LCD_DisplayNum ((*Minutes) / 10, 3);
        }
        // Deal with hour
        if ((*Minutes) == 0)
        {
            (*Hours)++;
            *Hours = (*Hours) % 24;
            LCD_DisplayNum((*Hours) % 10, 4);
            if (((*Hours) % 10) == 0)
            {
                LCD_DisplayNum ((*Hours) / 10, 5);
            }
        }
    }
    LCD_DisplayDot(2);
    //P2OUT ^= BIT4;    // do not toggle P2.4  = LED to save power             // 12/09/2015
}

void initGpio(void)
{
    // Configure all GPIO to Output Low
    P1OUT = 0x00;P2OUT = 0x00;P3OUT = 0x00;P4OUT = 0x00;
    P5OUT = 0x00;P6OUT = 0x00;P7OUT = 0x00;P8OUT = 0x00;

    P1DIR = 0xFF;P2DIR = 0xFF;P3DIR = 0xFF;P4DIR = 0xFF;  
    P5DIR = 0xFF;P6DIR = 0xFF;P7DIR = 0xFF;P8DIR = 0xFF;

    // Disable the GPIO power-on default high-impedance mode
    // to activate previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;
}

/* Measure count result (capacitance) of each sensor*/
/* Routine setup for four sensors, not dependent on NUM_SEN value!*/

void measure_count(void)                                                        // 11/24/2015
{ 
    TA0CTL = TASSEL_3 + MC_2;               // TACLK, cont mode                 // 11/24/2015
    TA0CCTL1 = CM_3 + CCIS_2 + CAP;         // Pos&Neg,GND,Cap                  // 11/24/2015
    
    /*Configure Ports for relaxation oscillator*/
    //CAPSIO0CTL |= CAPTIOEN + CAPSIOPOSEL0 + CAPSIOPISEL0; //P1.1              // 11/24/2015
    CAPSIO0CTL |= CAPTIOEN + CAPTIOPOSEL_4 + CAPTIOPISEL_0; 
                                            // P4.0 for capacitive touch        // 11/24/2015
    
    /*Setup Gate Timer*/
    WDTCTL = WDT_meas_setting;              // WDT, ACLK, interval timer        // 11/24/2015
    SFRIE1 |= WDTIE;                        // enable WDT interrupt             // 11/24/2015
    TA0CTL |= TACLR;                        // Clear Timer_A TAR                // 11/24/2015
    //__bis_SR_register(LPM0_bits+GIE);       // Wait for WDT interrupt         // 11/24/2015
    __bis_SR_register(LPM3_bits+GIE);       // LPM3 Wait for WDT interrupt      // 12/09/2015
    TA0CCTL1 ^= CCIS0;                      // Create SW capture of CCR1        // 11/24/2015
    meas_cnt = TA0CCR1;                     // Save result                      // 11/24/2015
    WDTCTL = WDTPW + WDTHOLD;               // Stop watchdog timer              // 11/24/2015
    CAPSIO0CTL = 0;
}

/* Watchdog Timer interrupt service routine*/
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)            // 11/24/2015
#pragma vector=WDT_VECTOR                                                       // 11/24/2015
__interrupt void watchdog_timer(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(WDT_VECTOR))) watchdog_timer (void)              // 11/24/2015
#else
#error Compiler not supported!
#endif
{
    TA0CCTL1 ^= CCIS0;                      // Create SW capture of CCR1        // 11/24/2015
    __bic_SR_register_on_exit(LPM3_bits);   // Exit LPM3 on reti                // 11/24/2015
}
